function [MSEposition, MSEvelocity, meanESS, meanTime, meanTimeOpt] = mainCAPF(numMC)
% This is the main script for running Soft-Constrained (Auxiliary) Particle Filter
% Input: series number for random number generator
% Output: MSE position; mean ESS; calculation time; optimisation time;

% The system dynamic equation is assumed to have the form:
% x_k+1 = f(x_k) + w_k; 
% The observation equation:
% y_k = h(x_k) + v_k;
% w_k and v_k are Gaussian Noises with covariances Q and R, respectively.

% The soft-constraints are defined in the follow form:
% g_i(x_k) < C_i
% C_i is assumed to be an i.i.d. ramdom variable from a distribution. 


% ======= Configuration ===================================================

% -------------------------------------------------------------------------
% Global variable 
% -------------------------------------------------------------------------
global Q; % 
global R; %
global T;
global C;
global Beta; % 
global sigcon;

% -------------------------------------------------------------------------
% SIMULATION PARAMETERS
% -------------------------------------------------------------------------

N = 500; %250;  % Number of particles
Beta = 1; % Resampling threshold for ESS [0,1]

time = 0:0.2:10; % Simulation time
steps = length(time); % Simulation steps


% -------------------------------------------------------------------------
% INITIAL STATES FOR THE SYSTEM 
% -------------------------------------------------------------------------

T = 0.2; % sampling interval;

% For a system with n states, m inputs, p noises, q outputs, set:
% n = ; m = ; p = ; q = ;

% The system model used in this example a 2D constant velocity model.
% The model is defined in fDyn.m

n = 4;

% set up covariance matrix for process noise
stdV = 2;
q1 = stdV^2*T;
q2 = stdV^2*T;

Q = [T^3/3*q1,        0, T^2/2*q1,        0; 
            0, T^3/3*q2,        0, T^2/2*q2;
     T^2/2*q1,        0,   T*q1,        0;
            0, T^2/2*q2,        0,   T*q2];

% set up covariance matrix for observatin noise
R = diag([0.2e-3, 0.2e-3]);

% state constraints are defined in gCon.m

% Road boundaries are approximated by using two polynomials with parameters
% p3 = 5e-5; 
% p2 = -0.004;
% p1 = -0.2;
% p0 = 125;

C = [2.5 2.5 12.5]'; % constraints such that g(x) <= C, or g(x)-C <=0; width of the road;

% The uncertainty of constraint parameters follows expoential distributions.
% The parameter for Exponential distributions are:
sigcon = [1/0.25 1/0.25 1/1];


cholQ = chol(Q);
cholR = chol(R);

%==========================================================================
% camera position and orientation
sensor_az = 80;
sensor_el = 45;
sensor_pos = [0 0 100];


%==========================================================================

if ~exist('numMC','var')
    rng('default');    
else
    rng(numMC); % for Monte Carlo simulation to generate a fixed sequence of random variables
end






% -------------------------------------------------------------------------
% Data Storage
% -------------------------------------------------------------------------

xArr = zeros(n,steps); % true state trajectory
zArr = cell(1,steps); % measurements
yObv = cell(1,steps); % Observed position


xhatArr = zeros(n,steps); % Estimated state
MSEArr = zeros(n,steps); % Mean-Square-Error
infoArr = zeros(5,steps); % information from the filter
% =========================================================================



% ============ Load example trajecotry ====================================


load xArr.mat; 
% plot(xArr(1,:), xArr(2,:), 'b-'); axis equal
% hold on

% =========================================================================

% generate observation data
for ii = 1:steps
   
%     z = [];
    
    za_bearing = atan2(xArr(2,ii)-sensor_pos(2), xArr(1,ii)-sensor_pos(1));

    el_bearing = atan2(0-sensor_pos(3), sqrt((xArr(2,ii)-sensor_pos(2))^2+(xArr(1,ii)-sensor_pos(1))^2));

    z = [za_bearing; el_bearing] + cholR*randn(2,1);

%     % draw the measurement
%     TargetRng = (0-sensor_pos(3))/tan(z(2));
%     plot(TargetRng*cos(z(1)),TargetRng*sin(z(1)),'rs');
    
    zArr{1,ii} = z;

   
end

% plot(xArr(1,:),xArr(2,:),'k','LineWidth',2);
% hold on

% =========================================================================

% =========================================================================


% =========================================================================
% -------------------------------------------------------------------------
% Particle Initialisation
% -------------------------------------------------------------------------

Pinit = diag([10 10 2.5 2.5]);
xpart_int = (xArr(:,1)+[-5 10 -2 -2]')*ones(1,N) + chol(Pinit) * randn(n,N);


%==========================================================================

w = ones(1,N)/N; % weights

% =========================================================================



% ============ Filtering ==================================================

hx = [];
for k = 1:steps
%     k;
    y = zArr{k};
    
    if k == 1
        xpart = xpart_int;
    end
    
    
    [xpart, w, xest, info] = softConPF(k,xpart,w,y); % Run scPF
%     [xpart, w, xest, info] = softConAuxPF(k,xpart,w,y); % Run scAPF
% 

    % ---------------------------------------------------------------------
    % Recording Data
    xhatArr(:,k) = xest;
    
%     
%     delete(hx);
%     hx = plot(xpart(1,:),xpart(2,:),'b.');
%    
%     figure(1);
%     

    infoArr(1,k) = info.ess; % effictive sample size
    infoArr(2,k) = info.fitness; % number of particles within constraints
    infoArr(3,k) = info.time; % calculation time
    infoArr(4,k) = info.timeOpt; % optimisation time
    % ---------------------------------------------------------------------
    
end




% =========================================================================


errx = xArr - xhatArr;
error2 = errx.^2; 
MSE=mean(error2,2);
MSEposition = (MSE(1,1)+MSE(2,1))/2;
MSEvelocity = (MSE(3,1)+MSE(4,1))/2;

meanESS = mean(infoArr(1,:));
meanTime = mean(infoArr(3,:));
meanTimeOpt = mean(infoArr(4,:));



